<?php if (!defined('BASEPATH')) die('No direct script access allowed');

/**
 * fieldtype
 *
 * @package     Currency fieldtype module
 * @category    Modules
 * @author      Rein de Vries <info@reinos.nl>
 * @link        http://reinos.nl
 * @copyright   Copyright (c) 2016 Reinos.nl Internet Media
 */

include(PATH_THIRD.'currency/config.php');


class Currency_ft extends EE_Fieldtype
{

    public $info = array(
        'name'      => CURRENCY_NAME,
        'version'   => CURRENCY_VERSION,
        'has_global_settings' => 'n'
    );

    public $settings = array();

    public $default_settings = array();

    public $has_array_data = TRUE;

    private $prefix;

    // ----------------------------------------------------------------------

    /**
     * Constructor
     *
     * @access public
     *
     * Calls the parent constructor
     */
    public function __construct()
    {
       parent::__construct();

        $this->prefix = CURRENCY_MAP.'_';

        //load lang file
        ee()->lang->loadfile(CURRENCY_MAP);

        //require settings
        require 'settings.php';
    }

    // ---------------------------------------------------------------------- 
    // Before saving the content to the database
    // ---------------------------------------------------------------------- 
    
    /**
     * save (Native EE)
     *
     * @access public
    */
    public function save($data)
    {
        return $this->_save($data);
    }

    // ---------------------------------------------------------------------- 

    /**
     * save (Low Variables)
     *
     * @access public
    */
//    public function save_var_field($data)
//    {
//        return $this->_save($data);
//    }

    // ----------------------------------------------------------------------

    /**
     * bulk_edit_save (Bulk Edit)
     *
     * @access public
     */
//    public function bulk_edit_save($data)
//    {
//        return $this->_save($data);
//    }

    // ----------------------------------------------------------------------

    /**
     * entry_api_save (Webservcie)
     *
     * @access public
     */
    public function webservice_save($data)
    {
        return $this->_save($data);
    }


    // ---------------------------------------------------------------------- 

    /**
     * save (Content Elements)
     *
     * @access public
    */
//    public function save_element($data)
//    {
//        return $this->_save($data);
//    }

    // ---------------------------------------------------------------------- 

    /**
     * save
     *
     * @access public
    */
    private function _save($data = '')
    {
        $data = str_replace($this->settings['currency_decimal_value'], '.', $data);

        //in grid situation, convert it to a decimal
        if($this->content_type == 'grid')
        {
            $data = number_format((double)$data, 2);
        }

        if(is_array($data))
        {
            $data = implode('|', $data);
        }

        //when we have a 0.00 value, reset it to null
        if($data == '')
        {
            return null;
        }

        return $data;
    }

    // ----------------------------------------------------------------------
    // Display the field for all types
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
    */
    function display_field($data)
    {
        if($this->content_type == 'grid')
        {
            $grid_table = 'channel_grid_field_'.$this->settings['grid_field_id'];
            if (ee()->db->table_exists($grid_table))
            {
                $fields = ee()->db->field_data($grid_table);

                foreach ($fields as $field)
                {
//                    if($field->name == 'col_id_'.$this->settings['col_id'] && $field->type != 'decimal')
//                    {
//                        //change the field tupe
                        ee()->db->query('ALTER TABLE  `exp_'.$grid_table.'` CHANGE  `col_id_'.$this->settings['col_id'].'`  `col_id_'.$this->settings['col_id'].'` decimal(65,'.$this->settings['currency_decimal'].') NULL DEFAULT NULL');

                        //update existing items to be null where they are zero
                        ee()->db->where('col_id_'.$this->settings['col_id'], '0'.$this->settings['currency_decimal'].'00');
                        ee()->db->update($grid_table, array(
                            'col_id_'.$this->settings['col_id'] => null
                        ));
//                    }
                }
            }
        }
        
        //generate the data
        return $this->_display_field($data);
    }

    // ----------------------------------------------------------------------

    /**
     * display_var_field (Low variables)
     *
     * @access public
    */
//    function display_var_field ($data)
//    {
//        //generate the data
//        return $this->_display_field($data, 'low_variables');
//    }

    // ----------------------------------------------------------------------
    
    /**
     * display_element (Content Elements)
     *
     * http://www.krea.com/docs/content-elements/element-development/ee2-functions-reference
     *
     * @access public
    */
//    function display_element($data)
//    {       
//        return $this->_display_field($data, 'content_elements');
//    }
    
    // ----------------------------------------------------------------------

    /**
     * display_field
     *
     * @access public
    */
    private function _display_field($data, $type = 'default')
    {
        //save data
        $data = trim($data);

        //fieldname
        switch($type)
        {
            case 'default':
            default : $field_name = $this->field_name;
                break;
        }

        $data = str_replace('.', $this->settings['currency_decimal_value'], $data);

        //return data to display
        return form_input($this->field_name, $data, 'id="'.$this->field_name.'"');
    }


    // ----------------------------------------------------------------------

    /**
     * Validates the field input (EE)
     *
     * @access public
     */
    public function validate($data)
    {
        return $this->_validate($data);
    }

    // ----------------------------------------------------------------------

    /**
     * Validates the field input (Bulk edit)
     *
     * @access public
     */
//    public function bulk_edit_validate($data)
//    {
//        return $this->validate($data);
//    }

    // ----------------------------------------------------------------------

    /**
     * Validates the field input (Webservice)
     *
     * @access public
     */
    public function webservice_validate($data)
    {
        return $this->validate($data);
    }

    // ----------------------------------------------------------------------

    /**
     * Validates the field input
     *
     * @access public
     */
    public function _validate($data)
    {
        $data = str_replace($this->settings['currency_decimal_value'], '.', $data);

        if(is_null($data) || empty($data) || is_int($data) || is_numeric($data) || (is_numeric( $data ) && floor( $data ) != $data))
        {
            return true;
        }

        return 'Not a currency';
    }

    // ----------------------------------------------------------------------
    // Replace the tags for all types
    // ----------------------------------------------------------------------

    /**
     * display_var_tag (Low variables)
     *
     * @access public
    */
//    public function display_var_tag($var_data, $tagparams, $tagdata)
//    {
//        return $this->replace_tag($var_data, $tagparams, $tagdata);
//    }

    // ----------------------------------------------------------------------

    /**
     * replace_element_tag (Content Elements)
     *
     * @access public
    */
//    public function replace_element_tag($data, $params = array(), $tagdata)
//    {
//        return $this->replace_tag($data, $params, $tagdata);
//    }

    // ----------------------------------------------------------------------

    /**
     * entry_api_pre_process
     *
     * @access public
     */
    public function webservice_pre_process($data = null)
    {
        return $this->replace_tag($data);
    }

    // ----------------------------------------------------------------------



    /**
     * replace_tag
     *
     * @access public
    */
    public function replace_tag($data, $params = array(), $tagdata = FALSE)
    {
        $data = trim($data);
        if ($data == FALSE) return;

        //set the decimal value
        $data = str_replace('.', $this->settings['currency_decimal_value'], $data);

        return $data;
    }



    // ----------------------------------------------------------------------
    
    /**
     * replace_tag_catchall
     *
     * @access public
    */
    /*function replace_tag_catchall($file_info, $params = array(), $tagdata = FALSE, $modifier)
    {
    
    }*/

    // ----------------------------------------------------------------------
    // Display the settings for all types
    // ----------------------------------------------------------------------
    
    /**
     * Display settings screen (Default EE)
     *
     * @access  public
     */
    function display_settings($data)
    {
        $settings_data = $this->_display_settings($data);

        if ($this->content_type() == 'grid')
        {
            return array('field_options_'.CURRENCY_MAP => $settings_data);
        }

        return array(
            'field_options_'.CURRENCY_MAP => array(
                'label' => 'field_options',
                'group' => CURRENCY_MAP,
                'settings' => $settings_data
            )
        );
    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (Matrix)
     *
     * @access  public
     */
//    function display_cell_settings($data)
//    {
//        return $this->_display_settings($data, array('matrix_input' => 'class="matrix-textarea"'));
//    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (Low variables)
     *
     * @access  public
     */
//    function display_var_settings($data)
//    {
//        return $this->_display_settings($data);
//    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (EE 2.7 GRID)
     *
     * @access  public
     */
//    function grid_display_settings($data)
//    {
//        return $this->_display_settings($data, array(), 'grid');
//    }

    // --------------------------------------------------------------------
    
    /**
     * Display settings screen (Content Elements)
     *
     * @access  public
     */
//    function display_element_settings($data)
//    {
//        return $this->_display_settings($data);
//    }

    // --------------------------------------------------------------------

     /**
     * Display settings screen
     *
     * @access  public
     */
    // --------------------------------------------------------------------

    /**
     * Display settings screen
     *
     * @access  public
     */
    private function _display_settings($data, $options = array(), $type = '')
    {
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                //global?
                if($options['global'])
                    continue;

                switch($options['type'])
                {
                    //multiselect
                    case 'm' :
                        break;

                    //select field
                    case 's' :
                        $return[] =  array(
                            'title' => $options['label'],
                            'desc' => isset($options['desc']) ? $options['desc'] : '',
                            'fields' => array(
                                $this->prefix.$options['name'] => array(
                                    'type' => 'select',
                                    'choices' => $options['options'],
                                    'value' => (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] ),
                                )
                            )
                        );

                        break;

                    //text field
                    default:
                    case 't' :
                        $return[] =  array(
                            'title' => $options['label'],
                            'desc' => isset($options['desc']) ? $options['desc'] : '',
                            'fields' => array(
                                $this->prefix.$options['name'] => array(
                                    'type' => 'text',
                                    'value' => (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] ),
                                )
                            )
                        );
                        break;
                }
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------
    // Save the settings for all types
    // ----------------------------------------------------------------------
    
    /**
     * save_settings (Default EE)
     *
     * @access public
    */
    function save_settings($data)
    {
        $settings = $this->_save_settings($data);

        if(isset($this->field_wide) && $this->field_wide)
        {
            $settings['field_wide'] = $this->field_wide;
        }

        return $settings;
    }

    // --------------------------------------------------------------------
    
    /**
     * save_settings ((Low variables))
     *
     * @access public
    */
//    function save_var_settings($data)
//    {  
//        return $this->_save_settings($data);
//    }  

    // --------------------------------------------------------------------
    
    /**
     * save_settings (EE GRID)
     *
     * @access public
    */
    function grid_save_settings($data)
    {
        return $this->_save_settings($data, true);
    }

    // --------------------------------------------------------------------

    /**
     * save_settings
     *
     * @access public
    */
    private function _save_settings($data, $use_data = false)
    { 
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                //global?
                if($options['global'])
                    continue;

                $return[$this->prefix.$options['name']] =  $use_data ? $data[$this->prefix.$options['name']] : ee()->input->post($this->prefix.$options['name']);
            }
        }

        return $return;
    }

    // --------------------------------------------------------------------

    /**
     * Do additional processing after the field is created/modified. (EE)
     *
     * @access public
     */
    public function post_save_settings($data)
    {
        $this->_post_save_settings($data);
    }

    // --------------------------------------------------------------------

    /**
     * Do additional processing after the field is created/modified.
     *
     * @access public
     */
    public function _post_save_settings($data)
    {
        //check if the field is converted to an int field
        $fields = ee()->db->field_data('channel_data');

        foreach ($fields as $field)
        {

            if($field->name == 'field_id_'.$data['field_id'] && $field->type != 'decimal')
            {
                //change the field tupe
                ee()->db->query('ALTER TABLE  `exp_channel_data` CHANGE  `field_id_'.$data['field_id'].'`  `field_id_'.$data['field_id'].'` decimal(65,'.$this->settings['currency_decimal'].') NULL DEFAULT NULL');

                //update existing items to be null where they are zero
                ee()->db->where('field_id_'.$data['field_id'], '0'.$this->settings['currency_decimal'].'00');
                ee()->db->update('channel_data', array(
                    'field_id_'.$data['field_id'] => null
                ));
            }
        }
    }

    // ----------------------------------------------------------------------
    
    /**
     * install
     *
     * @access public
    */
    function install()
    {
        $return = array();

        if(!empty($this->fieldtype_settings))
        {
            foreach($this->fieldtype_settings as $field=>$options)
            {
                $return[$this->prefix.$options['name']] = $options['def_value'];
            }
        }

        return $return;
    }

    // ----------------------------------------------------------------------

    /**
     * Update
     */
    function update()
    {
        return TRUE;
    }

    // ----------------------------------------------------------------------

    /**
     * display_global_settings
     *
     * @access public
    */
    function display_global_settings()
    {
        $data = array_merge($this->settings, $_POST);

        $form = '';
        foreach($this->fieldtype_settings as $field=>$options)
        {
            //global?
            if(!$options['global'])
                continue;

            switch($options['type'])
            {
                //multiselect
                case 'm' : 
                    $form .= form_label($options['label'], $options['label']).NBS.form_multiselect($this->prefix.$options['name'], $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                break;

                //select field
                case 's' : 
                    $form .= form_label($options['label'], $options['label']).NBS.form_dropdown($this->prefix.$options['name'], $options['options'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                break;

                //text field
                default:
                case 't' : 
                    $form .= form_label($options['label'], $options['label']).NBS.form_input($this->prefix.$options['name'], (isset($data[$this->prefix.$options['name']]) ? $data[$this->prefix.$options['name']] : $options['def_value'] )).NBS.NBS.NBS.' ';
                break;
            }
        }

        return $form;
    }

    // ----------------------------------------------------------------------

    /**
     * save_global_settings
     *
     * @access public
    */
    function save_global_settings()
    {
        return array_merge($this->settings, $_POST);
    }

    // ---------------------------------------------------------------------- 

    /**
     * accepts_content_type (GRID EE 2.7)
     *
     * @access public
    */
    public function accepts_content_type($name)
    {
        return ($name == 'channel' || $name == 'grid');
        //return true;
    }

    // --------------------------------------------------------------------

}
